<?php

//
// Advanced eZDOMNode and eZDOMDocument classes for compatibility with
// eZ publish versions prior to 3.8
//
// Created on: <18-Nov-2005 10:10:02 ks>
//
// Copyright (C) 1999-2006 eZ systems as. All rights reserved.
//


include_once( "lib/ezxml/classes/ezxml.php" );

class advDOMDocument extends eZDOMDocument
{

    /*
       Clean up memory. (destructor)
    */
    function cleanup()
    {
        if ( $this->Root )
            advDOMNode::cleanup( $this->Root );
    }

    // Class name replaced.

    function setRoot( &$node )
    {
       // if ( get_class( $node ) == "advdomnode" )
       // {
            $this->Root =& $node;
       // }
    }

    // \note W3C DOM function

    function appendChild( &$node )
    {
        //if ( get_class( $node ) == "advdomnode" )
       // {
            $this->Root =& $node;
       // }
    }

    function toString( $charset = true, $charsetConversion = true )
    {
        $charsetText = '';
        if ( $charset === true )
            $charset = 'UTF-8';
        if ( $charset !== false )
            $charsetText = " encoding=\"$charset\"";
        $text = "<?xml version=\"1.0\"$charsetText?>\n";

        if ( get_class( $this->Root ) == "advdomnode" )
        {
            $text .= $this->Root->toString( 0, $charset );
        }

        if ( $charsetConversion )
        {
            include_once( 'lib/ezi18n/classes/eztextcodec.php' );
            $codec =& eZTextCodec::instance( false, $charset, false );
            if ( $codec )
            {
                $text = $codec->convertString( $text );
            }
        }

        return $text;
    }

    // \note W3C DOM function

    function createTextNode( $text )
    {
        $text = preg_replace('/[\x00-\x08\x0b-\x0c\x0e-\x1f]/', '', $text);

        $node = new advDOMNode();
        $node->setName( "#text" );
        $node->setContent( $text );
        $node->setType( 3 );

        return $node;
    }

    // \note W3C DOM function

    function &createElement( $name )
    {
        $node = new advDOMNode();
        $node->setName( $name );
        $node->setType( 1 );

        $this->registerElement( $node );

        return $node;
    }

    // \note W3C DOM function

    function createAttributeNS( $namespaceURI, $qualifiedName )
    {
        list( $prefix, $name ) = explode( ':', $qualifiedName );

        $attr = new advDOMNode();
        $attr->setName( $name );
        $attr->setPrefix( $prefix );
        $attr->setNamespaceURI( $namespaceURI );
        $attr->setType( 2 );
        return $attr;
    }

    // \note W3C DOM function

    function createAttribute( $name )
    {
        $attr = new advDOMNode();
        $attr->setName( $name );
        $attr->setType( 2 );
        return $attr;
    }
}

class advDOMNode extends eZDOMNode
{
    function dn2( $node, $text )
    {
        if ( !$node )
            $node =& $this;
        
        $parentName = $node->parentNode->nodeName;
        unset( $node->parentNode );
        $node->parentNode = $parentName;
        $children = array();
        if ( count( $node->Children ) )
        {
            foreach( array_keys( $node->Children ) as $key )
            {
                $child =& $node->Children[$key];
                if ( $child->nodeName == '#text' )
                    $children[$key] = $child->content;
                else
                    $children[$key] = $child->nodeName;
            }
            unset( $node->Children );
            $node->Children = $children;
        }
        eZDebug::writeDebug( $node, $text );
    }


    function advDOMNode()
    {
        $this->Content =& $this->content;
        $this->Type =& $this->type;
        $this->nodeName =& $this->Name;
    }

    function cloneNode()
    {
        $tmp = new advDOMNode();
        $tmp->Name = $this->Name;
        $tmp->Type = $this->Type;
        $tmp->Content = $this->Content;
        $tmp->Children = null;
        $tmp->Attributes = $this->Attributes;
        $tmp->NamespaceURI = $this->NamespaceURI;
        $tmp->LocalName = $this->LocalName;
        $tmp->Prefix = $this->Prefix;

        $tmp->parentNode = null;
        return $tmp;
    }

    /*!
      Subtree destructor. Needed to clean memory properly.
    */
    function cleanup( &$node )
    {
        if ( $node->hasChildren() )
        {
            foreach( array_keys( $node->Children ) as $key )
            {
                $child =& $node->Children[$key];
                if ( $child->hasChildren() )
                {
                    $child->cleanup( $child );
                }

            }
            $node->removeChildren();
        }
    }

    function hasChildNodes()
    {
        return count( $this->Children ) > 0;
    }

    /*!
      Appends the node \a $node as a child of the current node.

      \return The node that was just inserted or \c false if it failed to insert a node.

      \note This will only make sense for element nodes.
      \note W3C DOM function
    */
    function appendChild( &$node )
    {
//        if ( get_class( $node ) == "advdomnode" )
//        {
            $node->parentNode =& $this;
            $this->Children[] =& $node;
            //return $node;
//        }
        //return false;
    }

    /*!
      Removes all child nodes from the current node.

      \note This will only make sense for element nodes.
    */
    function removeChildren()
    {
        foreach( array_keys( $this->Children ) as $key )
        {
           unset( $this->Children[$key]->parentNode );
           $this->Children[$key]->parentNode = null;
        }
        $this->Children = array();
    }


    /*!
      Removes child by the given child object.

      \note W3C DOM function
    */
    function removeChild( &$childToRemove )
    {
        unset( $childToRemove->parentNode );
        $childToRemove->parentNode = null;
        $child = $childToRemove;

        foreach ( array_keys( $this->Children ) as $key )
        {
            if ( $this->Children[$key]->parentNode === null )
            {
                unset( $this->Children[$key] );
                break;
            }
        }

        return $child;
    }

    /*!
      Replaces child by the new one given.

      \note W3C DOM function
    */
    function replaceChild( &$newChild, &$oldChild )
    {
        unset( $oldChild->parentNode );
        $oldChild->parentNode = null;

        $newChildren = array();

        foreach ( array_keys( $this->Children ) as $key )
        {
            if ( $this->Children[$key]->parentNode === null )
            {
                $newChild->parentNode =& $this;
                $newChildren[$key] =& $newChild;
            }
            else
            {
                $newChildren[$key] =& $this->Children[$key];
            }
        }
        $this->Children =& $newChildren;
        return $oldChild;
    }

    /*!
      Replaces child by the new one given.

      \note W3C DOM function
    */
    function insertBefore( &$newNode, &$refNode )
    {
        unset( $refNode->parentNode );
        $refNode->parentNode = null;

        $newChildren = array();

        foreach ( array_keys( $this->Children ) as $key )
        {
            if ( $this->Children[$key]->parentNode === null )
            {
                $newChildren[] =& $newNode;
                $newNode->parentNode =& $this;
                $refNode->parentNode =& $this;
            }
            $newChildren[] =& $this->Children[$key];
        }
        $this->Children =& $newChildren;
        return $newNode;
    }

    /*!
      \note emulation of W3C DOM property
    */

    function &nextSibling()
    {
        $ret = null;
        if ( !$this->parentNode )
            return $ret;

        $parent =& $this->parentNode;
        $this->flag = true;

        $next = false;
        $children =& $parent->Children;

        foreach( array_keys( $children ) as $child_key )
        {
            if ( $next )
            {
                $ret =& $children[$child_key];
                break;
            }
            elseif ( $children[$child_key]->flag === true )
            {
                $next = true;
            }
        }
        $this->flag = false;

        return $ret;
    }

   /*!
      \note emulation of W3C DOM property
    */

    function &previousSibling()
    {
        $ret = null;
        if ( !$this->parentNode )
            return $ret;

        $parent =& $this->parentNode;
        $this->flag = true;

        $prev = false;
        $children =& $parent->Children;
        foreach( array_keys( $children ) as $child_key )
        {
            if ( $prev !== false && $children[$child_key]->flag === true )
            {
                $ret =& $children[$prev];
                break;
            }
            $prev = $child_key;
        }
        $this->flag = false;
        return $ret;
    }

    // Class name replaced.

    function appendAttribute( &$node )
    {
        //if ( get_class( $node ) == "advdomnode" )
        //{
            $this->Attributes[] =& $node;
            return $node;
        //}
        return false;
    }

    // \note W3C DOM function

    function setAttribute( $name, $value )
    {
        foreach ( $this->Attributes as $attribute )
        {
            if ( $attribute->Name == $name && !$attribute->Prefix )
            {
                $attribute->setContent( $value );
                return $attribute;
            }
        }

        $attr = advDOMDocument::createAttribute( $name );
        $attr->setContent( $value );
        return $this->appendAttribute( $attr );
    }

    // \note W3C DOM function

    function setAttributeNS( $namespaceURI, $qualifiedName, $value )
    {
        foreach ( $this->Attributes as $attribute )
        {   
            if ( !$attribute->Prefix )
                continue;

            $fullName = $attribute->Prefix . ':' . $attribute->LocalName;
            if ( $fullName == $qualifiedName )
            {
                $attribute->setContent( $value );
                return $attribute;
            }
        }
        $attr = advDOMDocument::createAttributeNS( $namespaceURI, $qualifiedName );
        $attr->setContent( $value );
        return $this->appendAttribute( $attr );
    }

    // \note W3C DOM function
    function getAttribute( $attributeName )
    {
        $returnValue = '';
        foreach ( $this->Attributes as $attribute )
        {
            if ( $attribute->name() == $attributeName && !$attribute->Prefix )
                $returnValue = $attribute->Content;
        }
    
        return $returnValue;
    }

    // \note W3C DOM function
    function getAttributeNS( $namespaceURI, $localName )
    {
        $returnValue = '';
        foreach ( $this->Attributes as $attribute )
        {
            if ( $attribute->LocalName == $localName &&
                 $attribute->NamespaceURI == $namespaceURI )
                $returnValue = $attribute->Content;
        }
    
        return $returnValue;
    }

    // \note W3C DOM function
    function removeAttribute( $name )
    {
        $removed = false;
        foreach( array_keys( $this->Attributes ) as $key )
        {
            if ( $this->Attributes[$key]->name() == $name && !$this->Attributes[$key]->Prefix )
            {
                unset( $this->Attributes[$key] );
                $removed = true;
            }
        }
        return $removed;
    }

    // \note W3C DOM function
    function removeAttributeNS( $namespaceURI, $localName )
    {
        $removed = false;
        foreach( array_keys( $this->Attributes ) as $key )
        {
            if ( $this->Attributes[$key]->LocalName == $localName &&
                 $this->Attributes[$key]->NamespaceURI == $namespaceURI )
            {
                unset( $this->Attributes[$key] );
                $removed = true;
            }
        }
        return $removed;
    }


    /*!
      \return The first child of the node or \c false if there are no children.

      \note This will only make sense for element nodes.
      \note W3C DOM function
    */

    function &firstChild()
    {
        if ( count( $this->Children ) == 0 )
        {
            $child = false;
            return $child;
        }
        reset( $this->Children );
        $key = key( $this->Children );
        $child =& $this->Children[$key];

        return $child;
    }

    /*!
     \return The last child node or \c false if there are no children.

      \note This will only make sense for element nodes.
    */

    function &lastChild()
    {
        if ( count( $this->Children ) == 0 )
        {
            $child = false;
            return $child;
        }
        end( $this->Children );
        $key = key( $this->Children );
        $child =& $this->Children[$key];

        return $child;
    }

    /*!
      \return The content() of the first child node or \c false if there are no children.

      \note This will only make sense for element nodes.
      \sa elementTextContentByName
    */
    function textContent()
    {
        return $this->collectTextContent( $this );
    }

    function collectTextContent( &$element )
    {
        $ret = '';
        if ( $element->Type == EZ_XML_NODE_TEXT )
        {
            $ret = $element->content();
        }
        else
        {
            if ( count( $element->Children ) > 0 )
            {
                foreach( array_keys( $element->Children ) as $key )
                {
                    $child =& $element->Children[$key];
                    $ret .= $this->collectTextContent( $child );
                }
            }
        }
        return $ret;
    }

    /*!
      Outputs DOM subtree to the debug output in the easy readable form.
      
      \param node  subtree root node
    */

    function writeDebug( &$node, $text, $showAttributes = false, $showParent = false )
    {
        if ( !$node )
            $node =& $this;

        if ( $node )
        {
            if ( get_class( $node ) == 'advdomnode' )
            {
                $d = eZDOMNode::debugNode( $node, $showAttributes, $showParent );
                eZDebug::writeDebug( $d, $text );
            }
            else
                eZDebug::writeDebug( '$node has bad class', $text );
        }
        else 
            eZDebug::writeDebug( '$node is null', $text );
    }

    function debugNode( &$node, $showAttributes, $showParent )
    {
        $d = array();
        $d['name'] = $node->nodeName;
        if ( $node->nodeName == '#text' )
            $d['text'] = $node->content;
        else if ( $node->Type == 2 )
            $d['value'] = $node->value;

        if ( $showParent )
           $d['parent'] = $node->parentNode->nodeName;

        if ( count( $node->Children ) )
        {
            $d['children'] = array();
            foreach( array_keys($node->Children) as $child_key )
            {
                $d['children'][] = eZDOMNode::debugNode( $node->Children[$child_key], $showAttributes, $showParent );
            }
        }

        if ( $showAttributes && count( $node->Attributes ) )
        {
            $d['attributes'] = array();
            foreach( array_keys($node->Attributes) as $attr_key )
            {
                $d['attributes'][] = eZDOMNode::debugNode( $node->Attributes[$attr_key], $showAttributes, $showParent );
            }
        }
        return $d;
    }

    function writeDebugStr( &$node, $text )
    {
        if ( is_object( $node ) )
            eZDebug::writeDebug( $node->toString( 0 ), $text );
        else
            eZDebug::writeDebug( $node, $text );
    }

    var $parentNode = null;
//    var $nextSibling = null;
    var $nodeName = null;

    var $flag = false;
}

?>
